@@ -50,8 +50,12 @@
 #include <sys/poll.h>
 #include <sys/types.h>
 #include <errno.h>
+#if defined(__linux__)
 #include <linux/types.h>
 #include <linux/ioctl.h>
+#else
+#include "type_compat.h"
+#endif
 
 #ifdef SUPPORT_RESMGR
 #include <resmgr.h>
