@@ -107,6 +107,7 @@ static int snd_ctl_ext_elem_list(snd_ctl_t *handle, snd_ctl_elem_list_t *list)
 		ret = ext->callback->elem_list(ext, offset, ids);
 		if (ret < 0)
 			return ret;
+		ids->numid = offset + 1; /* fake number */
 		list->used++;
 		offset++;
 		ids++;
@@ -114,13 +115,24 @@ static int snd_ctl_ext_elem_list(snd_ctl_t *handle, snd_ctl_elem_list_t *list)
 	return 0;
 }
 
+static snd_ctl_ext_key_t get_elem(snd_ctl_ext_t *ext, snd_ctl_elem_id_t *id)
+{
+	int numid = id->numid;
+	if (numid > 0) {
+		ext->callback->elem_list(ext, numid - 1, id);
+		id->numid = numid;
+	} else
+		id->numid = 0;
+	return ext->callback->find_elem(ext, id);
+}
+
 static int snd_ctl_ext_elem_info(snd_ctl_t *handle, snd_ctl_elem_info_t *info)
 {
 	snd_ctl_ext_t *ext = handle->private_data;
 	snd_ctl_ext_key_t key;
 	int type, ret;
 
-	key = ext->callback->find_elem(ext, &info->id);
+	key = get_elem(ext, &info->id);
 	if (key == SND_CTL_EXT_KEY_NOT_FOUND)
 		return -ENOENT;
 	ret = ext->callback->get_attribute(ext, key, &type, &info->access, &info->count);
@@ -200,7 +212,7 @@ static int snd_ctl_ext_elem_read(snd_ctl_t *handle, snd_ctl_elem_value_t *contro
 	int type, ret;
 	unsigned int access, count;
 
-	key = ext->callback->find_elem(ext, &control->id);
+	key = get_elem(ext, &control->id);
 	if (key == SND_CTL_EXT_KEY_NOT_FOUND)
 		return -ENOENT;
 	ret = ext->callback->get_attribute(ext, key, &type, &access, &count);
@@ -254,7 +266,7 @@ static int snd_ctl_ext_elem_write(snd_ctl_t *handle, snd_ctl_elem_value_t *contr
 	int type, ret;
 	unsigned int access, count;
 
-	key = ext->callback->find_elem(ext, &control->id);
+	key = get_elem(ext, &control->id);
 	if (key == SND_CTL_EXT_KEY_NOT_FOUND)
 		return -ENOENT;
 	ret = ext->callback->get_attribute(ext, key, &type, &access, &count);
@@ -312,6 +324,51 @@ static int snd_ctl_ext_elem_unlock(snd_ctl_t *handle ATTRIBUTE_UNUSED,
 	return -ENXIO;
 }
 
+static int snd_ctl_ext_elem_tlv(snd_ctl_t *handle, int op_flag,
+				unsigned int numid,
+				unsigned int *tlv, unsigned int tlv_size)
+{
+	snd_ctl_ext_t *ext = handle->private_data;
+	snd_ctl_ext_key_t key;
+	int type, ret;
+	unsigned int access, count, len;
+	snd_ctl_elem_id_t id;
+
+	/* we don't support TLV on protocol ver 1.0.0 or earlier */
+	if (ext->version <= SNDRV_PROTOCOL_VERSION(1, 0, 0))
+		return -ENXIO;
+
+	snd_ctl_elem_id_clear(&id);
+	if (numid > 0) {
+		ext->callback->elem_list(ext, numid - 1, &id);
+		id.numid = numid;
+	} else
+		id.numid = 0;
+	key = ext->callback->find_elem(ext, &id);
+
+	if (key == SND_CTL_EXT_KEY_NOT_FOUND)
+		return -ENOENT;
+	ret = ext->callback->get_attribute(ext, key, &type, &access, &count);
+	if (ret < 0)
+		return ret;
+
+	if ((op_flag == 0 && (access & SND_CTL_EXT_ACCESS_TLV_READ) == 0) ||
+	    (op_flag > 0 && (access & SND_CTL_EXT_ACCESS_TLV_WRITE) == 0) ||
+	    (op_flag < 0 && (access & SND_CTL_EXT_ACCESS_TLV_COMMAND) == 0))
+		return -ENXIO;
+	if (access & SNDRV_CTL_ELEM_ACCESS_TLV_CALLBACK) {
+		return ext->tlv.c(ext, key, op_flag, numid, tlv, tlv_size);
+	} else {
+		if (op_flag)
+			return -ENXIO;
+		len = ext->tlv.p[1] + 2 * sizeof(unsigned int);
+		if (tlv_size < len)
+			return -ENOMEM;
+		memcpy(tlv, ext->tlv.p, len);
+		return 0;
+	}
+}
+
 static int snd_ctl_ext_next_device(snd_ctl_t *handle ATTRIBUTE_UNUSED,
 				   int *device ATTRIBUTE_UNUSED)
 {
@@ -417,6 +474,7 @@ static const snd_ctl_ops_t snd_ctl_ext_ops = {
 	.element_write = snd_ctl_ext_elem_write,
 	.element_lock = snd_ctl_ext_elem_lock,
 	.element_unlock = snd_ctl_ext_elem_unlock,
+	.element_tlv = snd_ctl_ext_elem_tlv,
 	.hwdep_next_device = snd_ctl_ext_next_device,
 	.hwdep_info = snd_ctl_ext_hwdep_info,
 	.pcm_next_device = snd_ctl_ext_next_device,
@@ -574,7 +632,7 @@ total number of control elements.  The elem_list returns the control element ID
 of the corresponding element offset (the offset is from 0 to elem_count - 1).
 The id field is initialized to all zero in prior to elem_list callback.  The callback
 has to fill the necessary field (typically iface, name and index) in return via the
-standard control API functions like #snd_ctl_elem_id_set_ifarce,
+standard control API functions like #snd_ctl_elem_id_set_interface,
 #snd_ctl_elem_id_set_name and #snd_ctl_elem_id_set_index, etc.  The callbacks should
 return 0 if successful, or a negative error code.
 
@@ -648,7 +706,8 @@ int snd_ctl_ext_create(snd_ctl_ext_t *ext, const char *name, int mode)
 	snd_ctl_t *ctl;
 	int err;
 
-	if (ext->version != SND_CTL_EXT_VERSION) {
+	if (ext->version < SNDRV_PROTOCOL_VERSION(1, 0, 0) ||
+	    ext->version > SND_CTL_EXT_VERSION) {
 		SNDERR("ctl_ext: Plugin version mismatch\n");
 		return -ENXIO;
 	}
