@@ -82,30 +82,6 @@ static int write_block_header(snd_tplg_t *tplg, unsigned int type,
 	return bytes;
 }
 
-static int write_data_block(snd_tplg_t *tplg, int size, int tplg_type,
-	const char *obj_name, void *data)
-{
-	int ret;
-
-	/* write the header for this block */
-	ret = write_block_header(tplg, tplg_type, 0,
-		tplg->version, 0, size, 1);
-	if (ret < 0) {
-		SNDERR("error: failed to write %s block %d\n", obj_name, ret);
-		return ret;
-	}
-
-	verbose(tplg, " %s : write %d bytes\n", obj_name, size);
-
-	ret = write(tplg->out_fd, data, size);
-	if (ret < 0) {
-		SNDERR("error: failed to write %s %d\n", obj_name, ret);
-		return ret;
-	}
-
-	return 0;
-}
-
 static int write_elem_block(snd_tplg_t *tplg,
 	struct list_head *base, int size, int tplg_type, const char *obj_name)
 {
@@ -221,13 +197,16 @@ static int write_block(snd_tplg_t *tplg, struct list_head *base,
 			SND_SOC_TPLG_TYPE_PCM, "pcm");
 	case SND_TPLG_TYPE_BE:
 		return write_elem_block(tplg, base, size,
-			SND_SOC_TPLG_TYPE_DAI_LINK, "be");
+			SND_SOC_TPLG_TYPE_BACKEND_LINK, "be");
 	case SND_TPLG_TYPE_CC:
 		return write_elem_block(tplg, base, size,
-			SND_SOC_TPLG_TYPE_DAI_LINK, "cc");
+			SND_SOC_TPLG_TYPE_CODEC_LINK, "cc");
 	case SND_TPLG_TYPE_DATA:
 		return write_elem_block(tplg, base, size,
 			SND_SOC_TPLG_TYPE_PDATA, "data");
+	case SND_TPLG_TYPE_DAI:
+		return write_elem_block(tplg, base, size,
+			SND_SOC_TPLG_TYPE_DAI, "dai");
 	default:
 		return -EINVAL;
 	}
@@ -317,6 +296,14 @@ int tplg_write_data(snd_tplg_t *tplg)
 		return ret;
 	}
 
+	/* write physical dai elems */
+	ret = write_block(tplg, &tplg->dai_list,
+		SND_TPLG_TYPE_DAI);
+	if (ret < 0) {
+		SNDERR("failed to write physical dai elems %d\n", ret);
+		return ret;
+	}
+
 	/* write be elems */
 	ret = write_block(tplg, &tplg->be_list,
 		SND_TPLG_TYPE_BE);
