@@ -160,7 +160,21 @@ static int tplg_build_widget(snd_tplg_t *tplg,
 
 	base = &elem->ref_list;
 
-	/* for each ref in this control elem */
+	/* A widget's private data sits before the embedded controls.
+	 * So merge the private data blocks at first
+	 */
+	 list_for_each(pos, base) {
+		ref = list_entry(pos, struct tplg_ref, list);
+
+		if (ref->type != SND_TPLG_TYPE_DATA)
+			continue;
+
+		err = tplg_copy_data(tplg, elem, ref);
+		if (err < 0)
+			return err;
+	}
+
+	/* Merge the embedded controls */
 	list_for_each(pos, base) {
 
 		ref = list_entry(pos, struct tplg_ref, list);
@@ -190,12 +204,6 @@ static int tplg_build_widget(snd_tplg_t *tplg,
 				err = copy_dapm_control(elem, ref->elem);
 			break;
 
-		case SND_TPLG_TYPE_DATA:
-			err = tplg_copy_data(tplg, elem, ref);
-			if (err < 0)
-				return err;
-			break;
-
 		default:
 			break;
 		}
@@ -507,6 +515,16 @@ int tplg_parse_dapm_widget(snd_tplg_t *tplg,
 			continue;
 		}
 
+		if (strcmp(id, "stream_name") == 0) {
+			if (snd_config_get_string(n, &val) < 0)
+				return -EINVAL;
+
+			elem_copy_text(widget->sname, val,
+				       SNDRV_CTL_ELEM_ID_NAME_MAXLEN);
+			tplg_dbg("\t%s: %s\n", id, val);
+			continue;
+		}
+
 		if (strcmp(id, "no_pm") == 0) {
 			if (snd_config_get_string(n, &val) < 0)
 				return -EINVAL;
