@@ -46,6 +46,10 @@ struct snd_soc_tplg_private *get_priv_data(struct tplg_elem *elem)
 		priv = &elem->widget->priv;
 		break;
 
+	case SND_TPLG_TYPE_DAI:
+		priv = &elem->dai->priv;
+		break;
+
 	default:
 		SNDERR("error: '%s': no support for private data for type %d\n",
 			elem->id, elem->type);
